#include <dht11.h> // Library for Temperature and Humidity Sensor

#define dht_apin A0 // Defines the Analog Pin A0 for use with Temperature and Humidity Sensor

dht11 DHT;

#include <Adafruit_GFX.h> // Adafruit's Graphic Effects Library
#include <Adafruit_NeoMatrix.h> // Adafruit's LED Matrix Library
#include <Adafruit_NeoPixel.h> // Adafruit's Individually Addressable LED Library
#ifndef PSTR // Checks to see if PSTR has been defined in this file or any of the included libraries
 #define PSTR 
#endif

#define PIN 6 // Digital Output Pin used for Data in on LED Matrix

Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(8, 8, PIN, // Code that describes the layout of the LED Matrix
  NEO_MATRIX_BOTTOM     + NEO_MATRIX_LEFT + // First LED pixel is in bottom left of matrix
  NEO_MATRIX_ROWS + NEO_MATRIX_ZIGZAG, // Matrix is built by rows, which go back and forth as the matrix is built
  NEO_GRB            + NEO_KHZ800); // Describes the color range and refresh rate of the matrix

const uint16_t colors[] = { // This is a constant array of colors used for the text displayed on the matrix, each color is made with a combination of RGB values
  matrix.Color(255, 0, 0), matrix.Color(255, 128, 0), matrix.Color(255, 255, 0), matrix.Color(128, 255, 0), matrix.Color(0, 255, 0), matrix.Color(0, 255, 255), matrix.Color(0, 128, 255), matrix.Color(0, 0, 255), matrix.Color(128, 0, 255), matrix.Color(255, 0, 255), matrix.Color(255, 255, 255) };

int tempF; // This is an integer so that I can convert Celsius to Fahrenheit later

void setup() {
  // put your setup code here, to run once:

  Serial.begin(9600); // Starts the Serial Monitor at 9600 bits per second, although the serial monitor is unused in the final version
  delay(500);// Delay half a second to let system boot
  Serial.println("DHT11 Humidity & temperature Sensor\n\n"); // Prints a title in the Serial Monitor, along with a line break to make it distinct
  delay(1000);// Waits one second before accessing Sensor

  matrix.begin();// Initializes the LED Matrix
  matrix.setTextWrap(false);
  matrix.setBrightness(10); // Sets the maximum brightness of the Matrix
  matrix.setTextColor(colors[5]); // Gives the text a color on startup
}

int x    = matrix.width();// This is an integer that is used to allow the text displayed on the Matrix to scroll horizontally
int counter = 60; // This is an integer that is used to seperate the sensor reading rate and the Matrix refresh rate

void loop() {
  // put your main code here, to run repeatedly:

   if(counter == 60){ // Checks to see if the counter has reached 55, which gives the text enough time to scroll without updating
    updateInfo(); // Calls to the function that outputs a Temp and Humidity reading, along with the appropriate color from the array
    counter = 0; // Resets the counter back to 0
   }
   
    matrix.fillScreen(0); //Clears the Matrix
  matrix.setCursor(x, 0); // Describes the starting pixel
  //matrix.print((DHT.temperature));
  matrix.print((tempF)); // Prints the calculated temperature in Fahrenheit, followed by F on the Matrix
  matrix.print(("F "));
  matrix.print((DHT.humidity));// Prints the sensed Humidity, followed by % on the Matrix
  matrix.print(("%"));
  if(--x < -46) { // Checks to see if the matrix's horizontal scroll has gone past 46 pixels, and if so, resets back to the beginning
    x = matrix.width();
  }
  matrix.show(); // Displays everything onto the matrix
  counter++; // Ticks the counter up one
  delay(100); // Delays the loop by .1 seconds 
}

void updateInfo(){ // Separates the Sensor reading rate from the loop, as to not overload the sensor
   DHT.read(dht_apin); // Gets a reading from the sensor using the defined pin
    
   // Serial.print("Current humidity = "); All of this part was used to output to the Serial Monitor before the Matrix was used
   // Serial.print(DHT.humidity);
   // Serial.print("%  ");
   // Serial.print("Temperature = ");
   // Serial.print(DHT.temperature); 
   // Serial.println("C  ");
 

    tempF = DHT.temperature * 9; // Takes the Temperature reading and multiplies it by nine
    tempF = tempF / 5; // Takes the above number and divides it by five
    tempF = tempF + 32; // Takes the above number and adds 32 to it to convert Celsius to Fahrenheit

    if(tempF >= 100){ // Checks to see if the calculated Fahrenheit reading is in a certain defined bracket, and assigns a color from the array to the pixels used in the Matrix
    matrix.setTextColor(colors[0]);// Greater than 100F = Red
    }
    if (tempF < 100 && tempF >= 90){ // Orange
      matrix.setTextColor(colors[1]);
    }
    else if (tempF < 90 && tempF >= 80){ // Yellow
      matrix.setTextColor(colors[2]);
    }
    else if (tempF < 80 && tempF >= 70){ // Lime
      matrix.setTextColor(colors[3]);
    }
    else if (tempF < 70 && tempF >= 60){ // Green
      matrix.setTextColor(colors[4]);
    }
    else if (tempF < 60 && tempF >= 50){ // Cyan
      matrix.setTextColor(colors[5]);
    }
    else if (tempF < 50 && tempF >= 40){ // Blue-Green
      matrix.setTextColor(colors[6]);
    }
    else if (tempF < 40 && tempF >= 30){ // Blue
      matrix.setTextColor(colors[7]);
    }
    else if (tempF < 30 && tempF >= 20){ // Indigo
      matrix.setTextColor(colors[8]);
    }
    else if (tempF < 20 && tempF >= 10){ // Purple
      matrix.setTextColor(colors[9]);
    }
    else if (tempF < 10){ 
      matrix.setTextColor(colors[10]); // Less than 10F = White
    }
}
